require 'test_helper'

class Gitmain::BlobTest < Minitest::Test
  def test_initialize_requires_repository
    assert_raises(ArgumentError) do
      Gitmain::Blob.new('', '')
    end
  end

  def test_initialize_requires_id
    assert_raises(ArgumentError) do
      empty_repository do |repository|
        Gitmain::Blob.new(repository, '')
      end
    end
  end

  def test_initialized_values_are_retrievable
    setup_repository do |repository|
      blob = Gitmain::Blob.new(repository, '3245b20dc91de6cd85203c7184430a969689ac56')
      assert_equal repository, blob.repository
      assert_equal '3245b20dc91de6cd85203c7184430a969689ac56', blob.id
    end
  end

  def test_changed_attributes
    setup_repository do |repository|
      blob = Gitmain::Blob.new(repository, '3245b20dc91de6cd85203c7184430a969689ac56')
      assert_equal({}, blob.changed_attributes)
      blob.content = 'Hello Gitmain World!'
      assert_equal({ content: 'Hello World.' }, blob.changed_attributes)
    end
  end

  def test_content
    setup_repository do |repository|
      blob = Gitmain::Blob.new(repository, '3245b20dc91de6cd85203c7184430a969689ac56')
      assert_equal 'Hello World.', blob.content
    end
  end

  def test_setting_content
    setup_repository do |repository|
      blob = Gitmain::Blob.new(repository, '3245b20dc91de6cd85203c7184430a969689ac56')
      blob.content = 'Hello Gitmain World!'
      assert_equal 'Hello Gitmain World!', blob.content
    end
  end

  def test_file_name
    setup_repository do |repository|
      blob = Gitmain::Blob.new(repository, '5eddd8887f867c74cef97aa385448b56554b3454', 'lib', 'project.rb')
      assert_equal 'lib/project.rb', blob.file_name
    end
  end
end
