require 'test_helper'

class Gitmain::TreeTest < Minitest::Test
  def test_initialize_requires_repository
    assert_raises(ArgumentError) do
      Gitmain::Tree.new('', '')
    end
  end

  def test_initialize_requires_id
    assert_raises(ArgumentError) do
      empty_repository do |repository|
        Gitmain::Tree.new(repository, '')
      end
    end
  end

  def test_initialized_values_are_retrievable
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, repository.tree_id)
      assert_equal repository, tree.repository
      assert_equal repository.tree_id, tree.id
    end
  end

  def test_blobs
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, repository.tree_id)
      blobs = tree.blobs
      assert_equal 1, blobs.length
      assert_blob blobs[0], repository, '3245b20dc91de6cd85203c7184430a969689ac56', 'README.md', nil
    end
  end

  def test_blobs_recursively
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, repository.tree_id)
      blobs = tree.blobs(recursive: true)
      assert_equal 6, blobs.length
      assert_blob blobs[0], repository, '3245b20dc91de6cd85203c7184430a969689ac56', 'README.md', nil
      assert_blob blobs[1], repository, '5eddd8887f867c74cef97aa385448b56554b3454', 'project.rb', 'lib'
      assert_blob blobs[2], repository, '57f650530905c305e7e84554974e5c635fa89c14', 'module.rb', 'lib/project'
      assert_blob blobs[3], repository, '4ffb2ae6bc8d01fc3e1e91291af58ae7e3f1d12f', 'module_test.rb', 'test/project'
      assert_blob blobs[4], repository, 'f9d5f423b12dceacb756ac6770fba6267d732d8b', 'other_test.rb', 'test/project/nested'
      assert_blob blobs[5], repository, 'e912ebf3225ea67b4612e431f6eb9805615544a7', 'test_helper.rb', 'test'
    end
  end

  def test_trees
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, repository.tree_id)
      trees = tree.trees
      assert_equal 2, trees.length
      assert_tree trees[0], repository, 'dee8c1e1f54639d65453b72e2c698e2b84be8954', 'lib', nil
      assert_tree trees[1], repository, 'd8280db63f0294724e2f95c371324be626fc3656', 'test', nil
    end
  end

  def test_trees_recursively
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, repository.tree_id)
      trees = tree.trees(recursive: true)
      assert_equal 5, trees.length
      assert_tree trees[0], repository, '3eec4e80e151a2b5ddbd688d4cd5d4cd6b09cfdc', 'project', 'lib'
      assert_tree trees[1], repository, 'dee8c1e1f54639d65453b72e2c698e2b84be8954', 'lib', nil
      assert_tree trees[2], repository, '6f8ea40fcbd86bf53c514ec7e2246d468dfb2b80', 'nested', 'test/project'
      assert_tree trees[3], repository, 'c062683857593d744b3e03671fdcf344cb4ab699', 'project', 'test'
      assert_tree trees[4], repository, 'd8280db63f0294724e2f95c371324be626fc3656', 'test', nil
    end
  end

  def test_non_root_blobs
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, 'd8280db63f0294724e2f95c371324be626fc3656', nil, 'test')
      blobs = tree.blobs
      assert_equal 1, blobs.length
      assert_blob blobs[0], repository, 'e912ebf3225ea67b4612e431f6eb9805615544a7', 'test_helper.rb', 'test'
    end
  end

  def test_non_root_blobs_recursively
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, 'd8280db63f0294724e2f95c371324be626fc3656', nil, 'test')
      blobs = tree.blobs(recursive: true)
      assert_equal 3, blobs.length
      assert_blob blobs[0], repository, '4ffb2ae6bc8d01fc3e1e91291af58ae7e3f1d12f', 'module_test.rb', 'test/project'
      assert_blob blobs[1], repository, 'f9d5f423b12dceacb756ac6770fba6267d732d8b', 'other_test.rb', 'test/project/nested'
      assert_blob blobs[2], repository, 'e912ebf3225ea67b4612e431f6eb9805615544a7', 'test_helper.rb', 'test'
    end
  end

  def test_non_root_trees
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, 'd8280db63f0294724e2f95c371324be626fc3656', nil, 'test')
      trees = tree.trees
      assert_equal 1, trees.length
      assert_tree trees[0], repository, 'c062683857593d744b3e03671fdcf344cb4ab699', 'project', 'test'
    end
  end

  def test_non_root_trees_recursively
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, 'd8280db63f0294724e2f95c371324be626fc3656', nil, 'test')
      trees = tree.trees(recursive: true)
      assert_equal 2, trees.length
      assert_tree trees[0], repository, '6f8ea40fcbd86bf53c514ec7e2246d468dfb2b80', 'nested', 'test/project'
      assert_tree trees[1], repository, 'c062683857593d744b3e03671fdcf344cb4ab699', 'project', 'test'
    end
  end

  def test_file_name
    setup_repository do |repository|
      tree = Gitmain::Tree.new(repository, 'c062683857593d744b3e03671fdcf344cb4ab699', 'test', 'project')
      assert_equal 'test/project', tree.file_name
    end
  end
end
