require_relative '../lib/gitmain'
require 'minitest'
require 'minitest/autorun'
require 'minitest/pride'
require 'rugged'

class Minitest::Test
  def assert_blob(blob, repository, id, name, path)
    assert blob.is_a?(Gitmain::Blob)
    assert_equal repository, blob.repository
    assert_equal id, blob.id
    assert_equal name, blob.name
    assert_equal path, blob.path
  end

  def assert_tree(tree, repository, id, name, path)
    assert tree.is_a?(Gitmain::Tree)
    assert_equal repository, tree.repository
    assert_equal id, tree.id
    assert_equal name, tree.name
    assert_equal path, tree.path
  end

  def repo_dirname
    File.expand_path('../../../tmp', __FILE__)
  end

  def setup
    FileUtils.rm_rf(repo_dirname)
    FileUtils.mkdir(repo_dirname)
  end

  def teardown
    FileUtils.rm_rf(repo_dirname)
  end

  def empty_repository
    FileUtils.rm_rf(File.join(repo_dirname, 'repository.git'))
    Rugged::Repository.init_at(File.join(repo_dirname, 'repository.git'), :bare)
    Gitmain::Repository.dirname(repo_dirname)
    repository = Gitmain::Repository.new('repository')
    yield repository if block_given?
    FileUtils.rm_rf(File.join(repo_dirname, 'repository.git'))
  end

  def full_repository
    FileUtils.rm_rf(File.join(repo_dirname, 'full.git'))
    Rugged::Repository.init_at(File.join(repo_dirname, 'full.git'))
    Gitmain::Repository.dirname(repo_dirname)
    repository = Gitmain::Repository.new('full')
    yield repository if block_given?
    FileUtils.rm_rf(File.join(repo_dirname, 'full.git'))
  end

  def setup_repository(branch = 'master')
    FileUtils.rm_rf(File.join(repo_dirname, 'repository.git'))
    Rugged::Repository.init_at(File.join(repo_dirname, 'repository.git'), :bare)
    FileUtils.rm_rf(File.join(repo_dirname, 'full_repository'))
    FileUtils.mkdir_p(File.join(repo_dirname, 'full_repository'))
    repo = Rugged::Repository.init_at(File.join(repo_dirname, 'full_repository'))
    index = repo.index
    oid = repo.write('Hello World.', :blob)
    index.add(path: 'README.md', oid: oid, mode: 0100644)
    oid = repo.write('# Project Setup', :blob)
    index.add(path: 'lib/project.rb', oid: oid, mode: 0100644)
    oid = repo.write('# Project Module', :blob)
    index.add(path: 'lib/project/module.rb', oid: oid, mode: 0100644)
    oid = repo.write('# Test Helper', :blob)
    index.add(path: 'test/test_helper.rb', oid: oid, mode: 0100644)
    oid = repo.write('# Module Test', :blob)
    index.add(path: 'test/project/module_test.rb', oid: oid, mode: 0100644)
    options = {}
    options[:tree] = index.write_tree(repo)
    options[:author] = options[:committer] = { email: 'gitmain@local', name: 'Gitmain', time: Time.new(2016, 2, 15, 1, 2, 3) }
    options[:message] = 'Full Repository Commit'
    options[:parents] = []
    options[:update_ref] = 'refs/heads/master'
    commit_id = Rugged::Commit.create(repo, options)
    index = repo.index
    oid = repo.write('# Nested Other Test', :blob)
    index.add(path: 'test/project/nested/other_test.rb', oid: oid, mode: 0100644)
    options = {}
    options[:tree] = index.write_tree(repo)
    options[:author] = options[:committer] = { email: 'gitmain@local', name: 'Gitmain', time: Time.new(2016, 2, 15, 1, 2, 3) }
    options[:message] = 'Second Commit'
    options[:parents] = [commit_id]
    options[:update_ref] = 'refs/heads/master'
    commit_id = Rugged::Commit.create(repo, options)
    remote = repo.remotes.create_anonymous "file://#{File.join(repo_dirname, 'repository.git')}"
    remote.push 'refs/heads/master'
    Gitmain::Repository.dirname(repo_dirname)
    repository = Gitmain::Repository.new('repository', branch)
    yield repository if block_given?
    FileUtils.rm_rf(File.join(repo_dirname, 'repository.git'))
    FileUtils.rm_rf(File.join(repo_dirname, 'full_repository.git'))
  end
end
